from pybacen.bacen.time_series import read_time_series, read_bacen_code, line_plot
from pybacen.bacen.open_data import read_bacen_complaints
from pybacen.bacen.bacen_data import Bacen_time_series, Bacen_data


from pybacen.bacen.bacen_data import Bacen_data as bd

def read_bacen_complaints(year: int, periodicity: str, period: str):
    return bd().read_bacen_complaints(year, periodicity, period)


"""
def read_list_reports():
    return bd().read_list_reports()

def read_registration_inst(limit: int = 100, filter: str = None, select: str = None):
    return bd().read_registration_inst(limit, filter, select)

def read_report_inst(year_month: str, type_inst: int, num_report: str, limit: int = 100, filter: str = None, select: str = None):
    return bd().read_report_inst(year_month, type_inst, num_report, limit, filter, select)
"""



import pandas as pd

try:
    pd.options.display.max_rows = 999
    pd.set_option('display.max_colwidth', None)
except:
    pass

class Investment_funds:

    def read_funds_quote(self, start: str = None, end: str = None) -> pd.core.frame.DataFrame:

        date_ranges = pd.date_range(start[0:7], end[0:7], freq='MS')

        full_fq = pd.DataFrame()

        for _date in date_ranges:
            url = f'http://dados.cvm.gov.br/dados/FI/DOC/INF_DIARIO/DADOS/inf_diario_fi_{_date.year}{_date.month}.csv'
            fq = pd.read_csv(url, sep=';')
            fq['DT_COMPTC'] = pd.to_datetime(fq['DT_COMPTC'], format=('%Y-%m-%d'))

            full_fq = pd.concat([full_fq, fq], ignore_index=True)
        
        start = start if start is not None else full_fq['DT_COMPTC'].min()
        end = end if end is not None else full_fq['DT_COMPTC'].max()

        full_fq = full_fq[(full_fq['DT_COMPTC']>=start) & (full_fq['DT_COMPTC']<=end)].copy()
         
        #if as_index == True:
        #    full_fq.set_index('DT_COMPTC', inplace=True)
        #else:
        #    pass
        
        return full_fq

    def read_registration_funds(self, start: str = None, end: str = None) -> pd.core.frame.DataFrame:
        url = f'http://dados.cvm.gov.br/dados/FI/CAD/DADOS/cad_fi.csv'
        
        rf = pd.read_csv(url, sep=';', encoding='cp1252')
        return rf


import json
import requests
import re
import pandas as pd
from pandas import DataFrame, to_datetime
from plotly.graph_objects import Figure, Candlestick, Line, Scatter
from plotly.express import box
import warnings


try:
    pd.options.display.max_rows = 999
    pd.set_option('display.max_colwidth', None)
except:
    pass

class Stock_quote:

    def __init__(self):
        self.user_agent = {"User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/95.0.4638.69 Safari/537.36"}
        self.quote_pattern_json = r"root\.App\.main = (.*?);\n}\(this\)\);"

    def read_stock_quote(self, stock_code: str or list, start: str = None, end: str = None, as_index: bool = True, pivot_table: bool = False) -> pd.core.frame.DataFrame:
        
        full_quote = pd.DataFrame()

        type_validation = lambda x: [x] if type(x) == str else x if type(x) == list else None

        stock_code = type_validation(stock_code)

        if stock_code is not None:
            for stock in stock_code:
                url = f'https://finance.yahoo.com/quote/{stock}/history'
                    
                response = requests.get(url, headers = self.user_agent)

                response = json.loads(re.search(self.quote_pattern_json, response.text, re.DOTALL).group(1))

                reponse = response["context"]["dispatcher"]["stores"]["HistoricalPriceStore"]

                quote = DataFrame(reponse['prices'])

                quote['date'] = to_datetime(to_datetime(quote["date"], unit="s").dt.date)

                quote = quote[quote['adjclose'].isnull() == False]

                start = start if start is not None else quote['date'].min()
                end = end if end is not None else quote['date'].max()

                quote = quote[(quote['date']>=start) & (quote['date']<=end)].copy()

                quote['stock_code'] = stock

                quote = quote[['date', 'stock_code', 'open', 'high', 'low', 'close', 'volume', 'adjclose']]

                full_quote = full_quote.append(quote, ignore_index=True)
            
            full_quote = full_quote.sort_values(by=['date', 'stock_code'])

            if pivot_table == True:
                full_quote = pd.pivot_table(full_quote, values=['open', 'high', 'low', 'close', 'adjclose'], index=['date'],
                        columns=['stock_code'], aggfunc= lambda x: x)
            else:
                if as_index == True:
                    full_quote.set_index('date', inplace=True)
                    

            return full_quote

        print('Error: Verify your stock_code: str or list')

    def candlestick(self, df: pd.core.frame.DataFrame, mov_avg: dict = None, template: str='plotly_dark'):
        warnings.filterwarnings("ignore")
        
        def get_candlestick(df, mov_avg):
            
            fig = Figure(Candlestick(x=df.index, open=df['open'], high=df['high'], low=df['low'], close=df['close']))
            fig.update_layout(xaxis_rangeslider_visible=False, 
                            template=template,
                            yaxis_title = 'Price', xaxis_title = 'Date', title= ''.join(df['stock_code'].unique())
                            )

            if mov_avg is not None:
                for i in mov_avg:
                    df[f'mov_avg_{i}'] = df['close'].rolling(window=i).mean()

                    fig.add_trace(
                        Scatter(
                        x=df.index,
                        y=df[f'mov_avg_{i}'],
                        line = dict(color = mov_avg[i]),
                        name = f'mov_avg_{i}'
                        )
                    )

            fig.show()
        
        for i in df['stock_code'].unique():
            get_candlestick(df[df['stock_code']== i], mov_avg)

    
    def describe(self, df: pd.core.frame.DataFrame, column_stats: str):
    
        df = df.groupby(['stock_code'], as_index=False).agg(
            min = pd.NamedAgg(column= column_stats, aggfunc=min),
            max = pd.NamedAgg(column= column_stats, aggfunc=max),
            avg = pd.NamedAgg(column= column_stats, aggfunc='mean'),
            count = pd.NamedAgg(column= column_stats, aggfunc='count'),
            q25 = pd.NamedAgg(column= column_stats, aggfunc= lambda x: x.quantile(0.25)),
            q50 = pd.NamedAgg(column= column_stats, aggfunc='median'),
            q75 = pd.NamedAgg(column= column_stats, aggfunc= lambda x: x.quantile(0.75)),
            std = pd.NamedAgg(column= column_stats, aggfunc= 'std'),
            return_ = pd.NamedAgg(column= column_stats, aggfunc= lambda x: x[-1] / x[0] - 1)
        )

        df['std/avg'] = df['std'] / df['avg']

        return df

    def boxplot(self, df: pd.core.frame.DataFrame,  x: str = 'stock_code', y: str = 'adjclose', color: str = 'stock_code', template: str = 'plotly_dark'):
        return box(df, x = x, y = y, color = color, template = template)


from warnings import filterwarnings
import pandas as pd 
import requests
import json
from plotly.graph_objects import Figure, Scatter


try:
    pd.options.display.max_rows = 999
    pd.set_option('display.max_colwidth', None)
except:
    pass

class Bacen_time_series:

    def read_time_series(self, bacen_code: int, start: str = None, end: str = None, as_index: bool = True) -> pd.core.frame.DataFrame:
        url = f'http://api.bcb.gov.br/dados/serie/bcdata.sgs.{str(bacen_code)}/dados?formato=json'
        ts = pd.read_json(url)
        ts['date'] = pd.to_datetime(ts['data'], dayfirst=True)
        
        start = start if start is not None else ts['date'].min()
        end = end if end is not None else ts['date'].max()

        ts = ts[(ts['date']>=start) & (ts['date']<=end)].copy()
         
        if as_index == True:
            ts.set_index('date', inplace=True)
            ts.drop('data', inplace=True, axis=1)
        else:
            pass
        
        return ts
    
    def read_bacen_code(self, search_text: str, period: str = None, unit: str = None) -> pd.core.frame.DataFrame:
        result = pd.read_csv('https://raw.githubusercontent.com/RPCastro07/Bacen_Time_Series_Codes/main/series_bacen_codes.csv', encoding='cp1252')
        
        if search_text is not None:
            if search_text != '':
                for i in list(filter(lambda x: x != '', search_text.split('%'))):
                    if search_text != '':
                        result = result[result['NM_SERIE'].str.upper().str.contains(i.upper())].copy()

        if period != None and period != '':
            result = result[result['PERIODICIDADE'].str.strip().str.upper()==period.upper().strip()]
        
        if unit != None and unit != '':
            result = result[result['UNIDADE'].str.strip().str.upper()==unit.upper().strip()]

        return result
    
    def line_plot(self, dfs: list, title: str, xtitle: str = 'Date', ytitle: str = 'Values', template: str = 'plotly_dark'):
    
        filterwarnings("ignore")
        
        dates = dfs[0][0].index
        
        
        for iteration, i in enumerate(dfs):
            
            df = i[0][i[0].index.isin(dates)].copy()
            
            if iteration == 0:
                fig = Figure(Scatter(
                        x=df.index,
                        y=df['valor'],
                        line = dict(color = i[2]),
                        name = i[1]
                        ))
                
                fig.update_layout(xaxis_rangeslider_visible=False,
                                template = template,
                                yaxis_title = ytitle, 
                                xaxis_title = xtitle,
                                title = title
                                )
                    
            else:
                fig.add_trace(Scatter(
                                    x=df.index,
                                    y=df['valor'],
                                    line = dict(color = i[2]),
                                    name = i[1]
                                    )
                            )
        fig.show() 
            

class Bacen_data:

    def read_bacen_complaints(self, year: int, periodicity: str, period: int) -> pd.core.frame.DataFrame:
        url = f'https://www3.bcb.gov.br/rdrweb/rest/ext/ranking/arquivo?ano={year}&periodicidade={periodicity.upper()}&periodo={period}&tipo=Bancos+e+financeiras'
        
        bc = pd.read_csv(url, sep = ';', encoding='cp1252')

        return bc

    # IF Data - Olinda
    def read_list_reports(self) -> pd.core.frame.DataFrame:
        
        url = 'https://olinda.bcb.gov.br/olinda/servico/IFDATA/versao/v1/odata/ListaDeRelatorio()?%24format=json'
        
        response = requests.get(url)

        response = json.loads(response.content)

        lr = pd.DataFrame(response['value'])

        return lr

    def important_link_olinda(self):
        print('Documentation: https://olinda.bcb.gov.br/olinda/servico/IFDATA/versao/v1/documentacao')
        print('Filters: https://olinda.bcb.gov.br/olinda/servico/ajuda')
        print('Site IF.data: https://www3.bcb.gov.br/ifdata/')

    def read_registration_inst(self, limit: int = 100, filter: str = None, select: str = None) -> pd.core.frame.DataFrame:

        
        filter = f"&$filter={filter}'" if filter is not None else ''
        select = f"&$select={select}'" if filter is not None else ''

        url = f"https://olinda.bcb.gov.br/olinda/servico/IFDATA/versao/v1/odata/IfDataCadastro(AnoMes=@AnoMes)?@AnoMes=202106&$top={limit}{filter}&$orderby=NomeInstituicao%20asc&$format=json{select}"
        response = requests.get(url, headers={'Cache-Control': 'no-cache'})

        json_list = json.loads(response.content)

        ri = pd.DataFrame(json_list['value'])

        return ri

    def read_report_inst(self, year_month: str, type_inst: int, num_report: str, limit: int = 100, filter: str = None, select: str = None) -> pd.core.frame.DataFrame:
       

        filter = f"&$filter={filter}'" if filter is not None else ''
        select = f"&$select={select}'" if filter is not None else ''

        url = f"https://olinda.bcb.gov.br/olinda/servico/IFDATA/versao/v1/odata/IfDataValores(AnoMes=@AnoMes,TipoInstituicao=@TipoInstituicao,Relatorio=@Relatorio)?@AnoMes={year_month}&@TipoInstituicao={type_inst}&@Relatorio='{num_report}'&$top={limit}{filter}&$format=json{select}"

        resp = requests.get(url, headers={'Cache-Control': 'no-cache', 'Pragma': 'no-cache'})

        json_list = json.loads(resp.content)

        rep = pd.DataFrame(json_list['value'])

        return rep




from pybacen.bacen.bacen_data import Bacen_time_series as bc_ts

def read_time_series(bacen_code: int, start: str = None, end: str = None, as_index: bool = True):
    return bc_ts().read_time_series(bacen_code, start, end, as_index)

def read_bacen_code(search_text: str, period: str = None, unit: str = None):
    return bc_ts().read_bacen_code(search_text, period, unit)

def line_plot(dfs: list, title: str, xtitle: str= 'Date', ytitle: str= 'Values', template: str='plotly_dark'):
   return bc_ts().line_plot(dfs, title, xtitle, ytitle, template)


from pybacen.cvm.cvm_investment_funds import Investment_funds as fi

def read_funds_quote(start: str = None, end: str = None):
    return fi().read_funds_quote(start, end)

def read_registration_funds():
    return fi().read_registration_funds()


import setuptools
from setuptools import setup, find_packages
import pathlib

here = pathlib.Path(__file__).parent.resolve()

#long_description = (here / 'README.md').read_text(encoding='utf-8')

setup(
 
    name='pybacen',  # Required
    version='1.1.1',  # Required
    description='Economic analysis in the Brazilian scenario',  # Optional
    #long_description=long_description,
    author='Rafael Pereira de Castro',  # Optional
    author_email='rafael.castro07@outlook.com',  # Optional

    license = 'MIT',
    classifiers=[  # Optional
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        "Programming Language :: Python :: 3.10",
        'Programming Language :: Python :: 3 :: Only',
    ],

    keywords='bacen, pybacen, series bacen',  # Optional

    packages = setuptools.find_packages(),

    python_requires='>=3.6, <4',


    install_requires=['pandas', 'requests', 'plotly'],  # Optional

)


from pybacen.yahoo_finance.stock_quote import Stock_quote as sq
import pandas as pd

def read_stock_quote(stock_code: str or list, start: str = None, end: str = None, as_index: bool = True, pivot_table: bool = True):
    return sq().read_stock_quote(stock_code, start, end, as_index, pivot_table)

def describe(df: pd.core.frame.DataFrame, column_stats: str):
    return sq().describe(df, column_stats)

def candlestick(df: pd.core.frame.DataFrame, mov_avg: dict = None, template: str='plotly_dark'):
    return sq().candlestick(df, mov_avg, template)

def boxplot(df: pd.core.frame.DataFrame,  x: str = 'stock_code', y: str = 'adjclose', color: str = 'stock_code', template: str = 'plotly_dark'):
    return sq().boxplot(df, x, y, color, template)




from pybacen.bacen.time_series import read_time_series, read_bacen_code, line_plot
from pybacen.bacen.open_data import read_bacen_complaints
from pybacen.bacen.bacen_data import Bacen_time_series, Bacen_data
from pybacen.yahoo_finance.stock_quote import Stock_quote
from pybacen.yahoo_finance.stocks import read_stock_quote, describe, candlestick, boxplot
from pybacen.cvm.cvm_investment_funds import Investment_funds
from pybacen.cvm.investment_funds import read_funds_quote, read_registration_funds



